@@ -15,7 +15,6 @@ from getpass import getpass
 from http.server import SimpleHTTPRequestHandler, ThreadingHTTPServer
 from io import BytesIO
 from multiprocessing import Pool
-from pathlib import Path
 from time import sleep
 
 import cv2
@@ -23,6 +22,7 @@ import imageio
 import numpy as np
 import pydicom
 import requests
+from pathy import Pathy
 from pydicom import uid
 from pydicom.encaps import encapsulate, generate_pixel_data_frame
 from pydicom.errors import InvalidDicomError
@@ -265,13 +265,18 @@ def package_dicom(ds, anonymize, compress):
     ds['PixelData'].is_undefined_length = True
 
 
+def size(path):
+    stat = path.stat()
+    return getattr(stat, 'size', getattr(stat, 'st_size', None))
+
+
 def wait_file(path):
-    path = Path(path)
+    path = Pathy.fluid(path)
     old = None
-    cur = os.path.getsize(path)
+    cur = size(path)
     while old != cur or not cur:
         sleep(1)
-        old, cur = cur, os.path.getsize(path)
+        old, cur = cur, size(path)
 
 
 def create_proxy(args):
@@ -407,7 +412,7 @@ class HandlerWorker:
 
     def process(self):
         try:
-            path = self.path and Path(self.path)
+            path = self.path and Pathy.fluid(self.path)
             ds = self.ds or pydicom.dcmread(self.f or path)
             params = self.file_params(ds, ae_title=self.ae_title, filename=getattr(path, 'stem', None))
             logger.info(f'processing {params}')
@@ -423,10 +428,10 @@ class HandlerWorker:
                 if dst:
                     src = self.args.src
                     rel = path.relative_to(src) if src else f"{ds.SOPInstanceUID}.dcm"
-                    out = (Path(dst) / rel).with_suffix(".dcm")
+                    out = (Pathy.fluid(dst) / rel).with_suffix(".dcm")
                     if self.args.overwrite or not out.is_file():
                         out.parent.mkdir(exist_ok=True, parents=True)
-                        ds.save_as(out)
+                        ds.save_as(out.open('wb'))
                 if hasattr(self.args, "auth"):
                     self.upload(ds, params)
                 extracted.add(k)
@@ -452,8 +457,9 @@ def handle_store(event, handler):
 
 
 def extracted_list(path):
-    if Path(path).is_file():
-        return {tuple(row) for row in csv.reader(open(path))}
+    path = path and Pathy.fluid(path)
+    if path and path.is_file():
+        return {tuple(row) for row in csv.reader(path.open())}
 
 
 def get_ip():
@@ -578,8 +584,8 @@ def read_args():
     )
     parser.add_argument(
         '--config-file',
-        default=Path("config.json"),
-        type=Path,
+        default=Pathy.fluid("config.json"),
+        type=Pathy.fluid,
         help='Configuration file for the EchoLoader',
     )
     parser.add_argument(
@@ -612,7 +618,7 @@ def read_args():
             for k, v in json.load(f).items():
                 setattr(args, k, v)
     args.env = (args.env and ENV(args.env)) or ENV('production')
-    args.src = args.src and Path(args.src)
+    args.src = args.src and Pathy.fluid(args.src)
     args.extracted = (args.extracted and extracted_list(args.extracted)) or set()
     args.sync = [PacsConnection(c) for c in args.sync]
     setup_logging(args.log, args.verbose)
@@ -629,7 +635,7 @@ def read_args():
 def main():
     args = read_args()
     handler = Handler(args)
-    sync = pacs = httpd = None
+    sync = observer = pacs = httpd = None
     try:
         if args.src:
             src = args.src
@@ -641,7 +647,7 @@ def main():
             if args.watch:
                 logger.warning(f"watching folder {os.path.abspath(src)}")
                 src.mkdir(exist_ok=True, parents=True)
-                observer = args.observer = PollingObserver()
+                observer = PollingObserver()
                 observer.schedule(handler, src, recursive=True)
                 observer.start()
         if args.sync:
@@ -660,12 +666,12 @@ def main():
             httpd = ThreadingHTTPServer(server_address=('0.0.0.0', args.proxy), RequestHandlerClass=create_proxy(args))
             thread = threading.Thread(target=httpd.serve_forever)
             thread.start()
-        while hasattr(args, 'observer') or handler.processing() or sync or pacs or httpd:
+        while observer or handler.processing() or sync or pacs or httpd:
             time.sleep(1)
     except KeyboardInterrupt:
         logger.warning("Interrupted, finishing up jobs")
     finally:
-        to_wait = [handler, getattr(args, 'observer', None), sync]
+        to_wait = [handler, observer, sync]
         to_wait = [e for e in to_wait if e]
         for e in to_wait:
             e.stop()
